@@ -24,17 +24,17 @@ import gi
 gi.require_version('Gtk', '3.0')
 from gi.repository import Gtk, Gdk, GObject, cairo
 from decimal import Decimal
-from electrum.util import print_error
-from electrum.bitcoin import is_valid
-from electrum import mnemonic, pyqrnative, WalletStorage, Wallet
+from electrum.util import print_error, InvalidPassword
+from electrum.bitcoin import is_valid, COIN
+from electrum.wallet import NotEnoughFunds
+from electrum import WalletStorage, Wallet
 
 Gdk.threads_init()
 APP_NAME = "Electrum"
 import platform
 MONOSPACE_FONT = 'Lucida Console' if platform.system() == 'Windows' else 'monospace'
 
-from electrum.util import format_satoshis
-from electrum.network import DEFAULT_SERVERS
+from electrum.util import format_satoshis, parse_URI
 from electrum.bitcoin import MIN_RELAY_TX_FEE
 
 def numbify(entry, is_int = False):
@@ -48,7 +48,7 @@ def numbify(entry, is_int = False):
             s = s.replace('.','')
             s = s[:p] + '.' + s[p:p+8]
         try:
-            amount = int( Decimal(s) * 100000000 )
+            amount = int(Decimal(s) * COIN)
         except Exception:
             amount = None
     else:
@@ -62,20 +62,16 @@ def numbify(entry, is_int = False):
 
 
 
-def show_seed_dialog(wallet, password, parent):
-    if not wallet.seed:
+def show_seed_dialog(seed, parent):
+    if not seed:
         show_message("No seed")
         return
-    try:
-        mnemonic = wallet.get_mnemonic(password)
-    except Exception:
-        show_message("Incorrect password")
-        return
+
     dialog = Gtk.MessageDialog(
         parent = parent,
-        flags = Gtk.DialogFlags.MODAL, 
-        buttons = Gtk.ButtonsType.OK, 
-        message_format = "Your wallet generation seed is:\n\n" + '"' + mnemonic + '"'\
+        flags = Gtk.DialogFlags.MODAL,
+        buttons = Gtk.ButtonsType.OK,
+        message_format = "Your wallet generation seed is:\n\n" + '"' + seed + '"'\
             + "\n\nPlease keep it in a safe place; if you lose it, you will not be able to restore your wallet.\n\n" )
     dialog.set_title("Seed")
     dialog.show()
@@ -84,10 +80,10 @@ def show_seed_dialog(wallet, password, parent):
 
 def restore_create_dialog():
 
-    # ask if the user wants to create a new wallet, or recover from a seed. 
+    # ask if the user wants to create a new wallet, or recover from a seed.
     # if he wants to recover, and nothing is found, do not create wallet
-    dialog = Gtk.Dialog("electrum", parent=None, 
-                        flags=Gtk.DialogFlags.MODAL|Gtk.DialogFlags.NO_SEPARATOR, 
+    dialog = Gtk.Dialog("electrum", parent=None,
+                        flags=Gtk.DialogFlags.MODAL,
                         buttons= ("create", 0, "restore",1, "cancel",2)  )
 
     label = Gtk.Label("Wallet file not found.\nDo you want to create a new wallet,\n or to restore an existing one?"  )
@@ -106,7 +102,7 @@ def run_recovery_dialog():
     message = "Please enter your wallet seed or the corresponding mnemonic list of words, and the gap limit of your wallet."
     dialog = Gtk.MessageDialog(
         parent = None,
-        flags = Gtk.DialogFlags.MODAL, 
+        flags = Gtk.DialogFlags.MODAL,
         buttons = Gtk.ButtonsType.OK_CANCEL,
         message_format = message)
 
@@ -125,7 +121,7 @@ def run_recovery_dialog():
     seed_box.pack_start(seed_entry, False, False, 10)
     add_help_button(seed_box, '.')
     seed_box.show()
-    vbox.pack_start(seed_box, False, False, 5)    
+    vbox.pack_start(seed_box, False, False, 5)
 
     dialog.show()
     r = dialog.run()
@@ -135,26 +131,21 @@ def run_recovery_dialog():
     if r==Gtk.ResponseType.CANCEL:
         return False
 
-    try:
-        seed.decode('hex')
-    except Exception:
-        print_error("Warning: Not hex, trying decode")
-        seed = mnemonic.mn_decode( seed.split(' ') )
-    if not seed:
-        show_message("no seed")
-        return False
-        
-    return seed
+    if Wallet.is_seed(seed):
+        return seed
+
+    show_message("no seed")
+    return False
 
 
 
 def run_settings_dialog(self):
 
     message = "Here are the settings of your wallet. For more explanations, click on the question mark buttons next to each input field."
-        
+
     dialog = Gtk.MessageDialog(
         parent = self.window,
-        flags = Gtk.DialogFlags.MODAL, 
+        flags = Gtk.DialogFlags.MODAL,
         buttons = Gtk.ButtonsType.OK_CANCEL,
         message_format = message)
 
@@ -173,14 +164,14 @@ def run_settings_dialog(self):
     fee_label.set_size_request(150,10)
     fee_label.show()
     fee.pack_start(fee_label,False, False, 10)
-    fee_entry.set_text( str( Decimal(self.wallet.fee) /100000000 ) )
+    fee_entry.set_text(str(Decimal(self.wallet.fee_per_kb) / COIN))
     fee_entry.connect('changed', numbify, False)
     fee_entry.show()
     fee.pack_start(fee_entry,False,False, 10)
     add_help_button(fee, 'Fee per kilobyte of transaction. Recommended value:0.0001')
     fee.show()
     vbox.pack_start(fee, False,False, 5)
-            
+
     nz = Gtk.HBox()
     nz_entry = Gtk.Entry()
     nz_label = Gtk.Label(label='Display zeros:')
@@ -194,22 +185,22 @@ def run_settings_dialog(self):
     add_help_button(nz, "Number of zeros displayed after the decimal point.\nFor example, if this number is 2, then '5.' is displayed as '5.00'")
     nz.show()
     vbox.pack_start(nz, False,False, 5)
-            
+
     dialog.show()
     r = dialog.run()
     fee = fee_entry.get_text()
     nz = nz_entry.get_text()
-        
+
     dialog.destroy()
     if r==Gtk.ResponseType.CANCEL:
         return
 
     try:
-        fee = int( 100000000 * Decimal(fee) )
+        fee = int(COIN * Decimal(fee))
     except Exception:
         show_message("error")
         return
-    self.wallet.set_fee(fee)
+    self.config.set_key('fee_per_kb', fee)
 
     try:
         nz = int( nz )
@@ -229,20 +220,17 @@ def run_settings_dialog(self):
 def run_network_dialog( network, parent ):
     image = Gtk.Image()
     image.set_from_stock(Gtk.STOCK_NETWORK, Gtk.IconSize.DIALOG)
+    host, port, protocol, proxy_config, auto_connect = network.get_parameters()
+    server = "%s:%s:%s"%(host, port, protocol)
     if parent:
         if network.is_connected():
-            interface = network.interface
-            status = "Connected to %s:%d\n%d blocks"%(interface.host, interface.port, network.blockchain.height())
+            status = "Connected to %s\n%d blocks"%(host, network.get_local_height())
         else:
             status = "Not connected"
     else:
         import random
         status = "Please choose a server.\nSelect cancel if you are offline."
 
-    if network.is_connected():
-        server = interface.server
-        host, port, protocol = server.split(':')
-
     servers = network.get_servers()
 
     dialog = Gtk.MessageDialog( parent, Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
@@ -250,7 +238,7 @@ def run_network_dialog( network, parent ):
     dialog.set_title("Server")
     dialog.set_image(image)
     image.show()
-    
+
     vbox = dialog.vbox
     host_box = Gtk.HBox()
     host_label = Gtk.Label(label='Connect to:')
@@ -303,15 +291,15 @@ def run_network_dialog( network, parent ):
     combobox.connect("changed", lambda x:set_protocol('tshg'[combobox.get_active()]))
     if network.is_connected():
         set_combobox(protocol)
-        
+
     server_list = Gtk.ListStore(str)
     for host in servers.keys():
         server_list.append([host])
-    
+
     treeview = Gtk.TreeView(model=server_list)
     treeview.show()
 
-    label = 'Active Servers' if network.irc_servers else 'Default Servers'
+    label = 'Active Servers' if network.is_connected() else 'Default Servers'
     tvcolumn = Gtk.TreeViewColumn(label)
     treeview.append_column(tvcolumn)
     cell = Gtk.CellRendererText()
@@ -353,13 +341,12 @@ def run_network_dialog( network, parent ):
 
     try:
         host, port, protocol = server.split(':')
-        proxy = network.config.get('proxy')
-        auto_connect = network.config.get('auto_cycle')
-        network.set_parameters(host, port, protocol, proxy, auto_connect)
     except Exception:
         show_message("error:" + server)
         return False
 
+    network.set_parameters(host, port, protocol, proxy_config, auto_connect)
+
 
 
 
@@ -367,8 +354,8 @@ def run_network_dialog( network, parent ):
 def show_message(message, parent=None):
     dialog = Gtk.MessageDialog(
         parent = parent,
-        flags = Gtk.DialogFlags.MODAL, 
-        buttons = Gtk.ButtonsType.CLOSE, 
+        flags = Gtk.DialogFlags.MODAL,
+        buttons = Gtk.ButtonsType.CLOSE,
         message_format = message )
     dialog.show()
     dialog.run()
@@ -431,7 +418,7 @@ def change_password_dialog(is_encrypted, parent):
     new_password = password_entry.get_text()
     new_password2 = password2_entry.get_text()
     dialog.destroy()
-    if result == Gtk.ResponseType.CANCEL: 
+    if result == Gtk.ResponseType.CANCEL:
         return
 
     if new_password != new_password2:
@@ -452,36 +439,36 @@ def add_help_button(hbox, message):
     hbox.pack_start(button,False, False, 0)
 
 
-class MyWindow(Gtk.Window): __gsignals__ = dict( mykeypress = (GObject.SignalFlags.RUN_LAST | GObject.SignalFlags.ACTION, None, (str,)) )
-
-GObject.type_register(MyWindow)
-#FIXME: can't find docs how to create keybindings in PyGI
-#Gtk.binding_entry_add_signall(MyWindow, Gdk.KEY_W, Gdk.ModifierType.CONTROL_MASK, 'mykeypress', ['ctrl+W'])
-#Gtk.binding_entry_add_signall(MyWindow, Gdk.KEY_Q, Gdk.ModifierType.CONTROL_MASK, 'mykeypress', ['ctrl+Q'])
-
-
 class ElectrumWindow:
 
     def show_message(self, msg):
         show_message(msg, self.window)
 
+    def on_key(self, w, event):
+        if Gdk.ModifierType.CONTROL_MASK & event.state and event.keyval in [113,119]:
+            Gtk.main_quit()
+        return True
+
     def __init__(self, wallet, config, network):
         self.config = config
         self.wallet = wallet
         self.network = network
         self.funds_error = False # True if not enough funds
         self.num_zeros = int(self.config.get('num_zeros',0))
-
-        self.window = MyWindow(Gtk.WindowType.TOPLEVEL)
+        self.window = Gtk.Window(Gtk.WindowType.TOPLEVEL)
+        self.window.connect('key-press-event', self.on_key)
         title = 'Electrum ' + self.wallet.electrum_version + '  -  ' + self.config.path
         if not self.wallet.seed: title += ' [seedless]'
         self.window.set_title(title)
         self.window.connect("destroy", Gtk.main_quit)
         self.window.set_border_width(0)
-        self.window.connect('mykeypress', Gtk.main_quit)
+        #self.window.connect('mykeypress', Gtk.main_quit)
         self.window.set_default_size(720, 350)
         self.wallet_updated = False
 
+        from electrum.util import StoreDict
+        self.contacts = StoreDict(self.config, 'contacts')
+
         vbox = Gtk.VBox()
 
         self.notebook = Gtk.Notebook()
@@ -493,7 +480,7 @@ class ElectrumWindow:
         self.create_about_tab()
         self.notebook.show()
         vbox.pack_start(self.notebook, True, True, 2)
-        
+
         self.status_bar = Gtk.Statusbar()
         vbox.pack_start(self.status_bar, False, False, 0)
 
@@ -515,7 +502,8 @@ class ElectrumWindow:
                     password = password_dialog(self.window)
                     if not password: return
                 else: password = None
-                show_seed_dialog(wallet, password, self.window)
+                seed = wallet.get_mnemonic(password)
+                show_seed_dialog(seed, self.window)
             button = Gtk.Button('S')
             button.connect("clicked", seedb, self.wallet )
             button.set_relief(Gtk.ReliefStyle.NONE)
@@ -590,7 +578,7 @@ class ElectrumWindow:
                         if to_address:
                             s = r + ' <' + to_address + '>'
                             GObject.idle_add( lambda: self.payto_entry.set_text(s) )
-                
+
 
         thread.start_new_thread(update_status_bar_thread, ())
         if self.wallet.seed:
@@ -611,7 +599,7 @@ class ElectrumWindow:
 
             try:
                 wallet.get_seed(password)
-            except Exception:
+            except InvalidPassword:
                 show_message("Incorrect password")
                 return
 
@@ -630,7 +618,7 @@ class ElectrumWindow:
 
 
     def create_send_tab(self):
-        
+
         page = vbox = Gtk.VBox()
         page.show()
 
@@ -690,23 +678,28 @@ class ElectrumWindow:
         payto_sig_id = Gtk.Label(label='')
         payto_sig.pack_start(payto_sig_id, False, False, 0)
         vbox.pack_start(payto_sig, True, True, 5)
-        
+
 
         self.user_fee = False
 
         def entry_changed( entry, is_fee ):
-            self.funds_error = False
             amount = numbify(amount_entry)
             fee = numbify(fee_entry)
             if not is_fee: fee = None
             if amount is None:
                 return
-            #assume two outputs - one for change
-            inputs, total, fee = self.wallet.choose_tx_inputs( amount, fee, 2 )
-            if not is_fee:
-                fee_entry.set_text( str( Decimal( fee ) / 100000000 ) )
-                self.fee_box.show()
-            if inputs:
+            coins = self.wallet.get_spendable_coins()
+            try:
+                tx = self.wallet.make_unsigned_transaction(coins, [('op_return', 'dummy_tx', amount)], self.config, fee)
+                self.funds_error = False
+            except NotEnoughFunds:
+                self.funds_error = True
+
+            if not self.funds_error:
+                if not is_fee:
+                    fee = tx.get_fee()
+                    fee_entry.set_text(str(Decimal(fee) / COIN))
+                    self.fee_box.show()
                 amount_entry.modify_text(Gtk.StateType.NORMAL, Gdk.color_parse("#000000"))
                 fee_entry.modify_text(Gtk.StateType.NORMAL, Gdk.color_parse("#000000"))
                 send_button.set_sensitive(True)
@@ -714,10 +707,9 @@ class ElectrumWindow:
                 send_button.set_sensitive(False)
                 amount_entry.modify_text(Gtk.StateType.NORMAL, Gdk.color_parse("#cc0000"))
                 fee_entry.modify_text(Gtk.StateType.NORMAL, Gdk.color_parse("#cc0000"))
-                self.funds_error = True
 
         amount_entry.connect('changed', entry_changed, False)
-        fee_entry.connect('changed', entry_changed, True)        
+        fee_entry.connect('changed', entry_changed, True)
 
         self.payto_entry = payto_entry
         self.payto_fee_entry = fee_entry
@@ -738,18 +730,15 @@ class ElectrumWindow:
             entry.modify_base(Gtk.StateType.NORMAL, Gdk.color_parse("#ffffff"))
 
     def set_url(self, url):
-        payto, amount, label, message, signature, identity, url = self.wallet.parse_url(url, self.show_message, self.question)
+        out = parse_URI(url)
+        address = out.get('address')
+        message = out.get('message')
+        amount = out.get('amount')
         self.notebook.set_current_page(1)
-        self.payto_entry.set_text(payto)
+        self.payto_entry.set_text(address)
         self.message_entry.set_text(message)
         self.amount_entry.set_text(amount)
-        if identity:
-            self.set_frozen(self.payto_entry,True)
-            self.set_frozen(self.amount_entry,True)
-            self.set_frozen(self.message_entry,True)
-            self.payto_sig_id.set_text( '      The bitcoin URI was signed by ' + identity )
-        else:
-            self.payto_sig.set_visible(False)
+        self.payto_sig.set_visible(False)
 
     def create_about_tab(self):
         from gi.repository import Pango
@@ -787,7 +776,7 @@ class ElectrumWindow:
 
         m1 = re.match('^(|([\w\-\.]+)@)((\w[\w\-]+\.)+[\w\-]+)$', r)
         m2 = re.match('(|([\w\-\.]+)@)((\w[\w\-]+\.)+[\w\-]+) \<([1-9A-HJ-NP-Za-km-z]{26,})\>', r)
-        
+
         if m1:
             to_address = self.wallet.get_alias(r, True, self.show_message, self.question)
             if not to_address:
@@ -805,12 +794,12 @@ class ElectrumWindow:
             return
 
         try:
-            amount = int( Decimal(amount_entry.get_text()) * 100000000 )
+            amount = int(Decimal(amount_entry.get_text()) * COIN)
         except Exception:
             self.show_message( "invalid amount")
             return
         try:
-            fee = int( Decimal(fee_entry.get_text()) * 100000000 )
+            fee = int(Decimal(fee_entry.get_text()) * COIN)
         except Exception:
             self.show_message( "invalid fee")
             return
@@ -823,17 +812,17 @@ class ElectrumWindow:
             password = None
 
         try:
-            tx = self.wallet.mktx( [(to_address, amount)], password, fee )
+            tx = self.wallet.mktx( [(to_address, amount)], password, self.config, fee)
         except Exception as e:
             self.show_message(str(e))
             return
 
-        if tx.requires_fee(self.wallet.verifier) and fee < MIN_RELAY_TX_FEE:
+        if tx.requires_fee(self.wallet) and fee < MIN_RELAY_TX_FEE:
             self.show_message( "This transaction requires a higher fee, or it will not be propagated by the network." )
             return
 
-            
-        if label: 
+
+        if label:
             self.wallet.labels[tx.hash()] = label
 
         status, msg = self.wallet.sendtx( tx )
@@ -865,7 +854,7 @@ class ElectrumWindow:
                 #        s = "self-signed"
                 #    msg = 'Alias: '+ m + '\nTarget address: '+ a[1] + '\n\nSigned by: ' + s + '\nSigning address:' + a[0]
                 #    self.show_message(msg)
-            
+
 
     def treeview_key_press(self, treeview, event):
         c = treeview.get_cursor()[0]
@@ -1033,14 +1022,15 @@ class ElectrumWindow:
             hbox.pack_start(button,False, False, 0)
 
         def showqrcode(w, treeview, liststore):
+            import qrcode
             path, col = treeview.get_cursor()
             if not path: return
             address = liststore.get_value(liststore.get_iter(path), 0)
-            qr = pyqrnative.QRCode(4, pyqrnative.QRErrorCorrectLevel.H)
-            qr.addData(address)
-            qr.make()
+            qr = qrcode.QRCode()
+            qr.add_data(address)
             boxsize = 7
-            boxcount_row = qr.getModuleCount()
+            matrix = qr.get_matrix()
+            boxcount_row = len(matrix)
             size = (boxcount_row + 4) * boxsize
             def area_expose_cb(area, cr):
                 style = area.get_style()
@@ -1050,7 +1040,7 @@ class ElectrumWindow:
                 Gdk.cairo_set_source_color(cr, style.black)
                 for r in range(boxcount_row):
                     for c in range(boxcount_row):
-                        if qr.isDark(r, c):
+                        if matrix[r][c]:
                             cr.rectangle((c + 2) * boxsize, (r + 2) * boxsize, boxsize, boxsize)
                             cr.fill()
             area = Gtk.DrawingArea()
@@ -1094,10 +1084,7 @@ class ElectrumWindow:
                 path, col = treeview.get_cursor()
                 if path:
                     address = liststore.get_value( liststore.get_iter(path), 0)
-                    if address in wallet.frozen_addresses:
-                        wallet.unfreeze(address)
-                    else:
-                        wallet.freeze(address)
+                    wallet.set_frozen_state([address], not wallet.is_frozen(address))
                     self.update_receiving_tab()
             button.connect("clicked", freeze_address, treeview, liststore, self.wallet)
             button.show()
@@ -1123,26 +1110,31 @@ class ElectrumWindow:
         return vbox
 
     def update_status_bar(self):
-        interface = self.network.interface
+
         if self.funds_error:
             text = "Not enough funds"
-        elif interface and interface.is_connected:
-            self.network_button.set_tooltip_text("Connected to %s:%d.\n%d blocks"%(interface.host, interface.port, self.network.blockchain.height()))
+        elif self.network.is_connected():
+            host, port, _,_,_ = self.network.get_parameters()
+            port = int(port)
+            height = self.network.get_local_height()
+            self.network_button.set_tooltip_text("Connected to %s:%d.\n%d blocks"%(host, port, height))
             if not self.wallet.up_to_date:
                 self.status_image.set_from_stock(Gtk.STOCK_REFRESH, Gtk.IconSize.MENU)
                 text = "Synchronizing..."
             else:
                 self.status_image.set_from_stock(Gtk.STOCK_YES, Gtk.IconSize.MENU)
-                self.network_button.set_tooltip_text("Connected to %s:%d.\n%d blocks"%(interface.host, interface.port, self.network.blockchain.height()))
-                c, u = self.wallet.get_balance()
-                text =  "Balance: %s "%( format_satoshis(c,False,self.num_zeros) )
-                if u: text +=  "[%s unconfirmed]"%( format_satoshis(u,True,self.num_zeros).strip() )
+                c, u, x = self.wallet.get_balance()
+                text = "Balance: %s "%(format_satoshis(c, False, self.num_zeros))
+                if u:
+                    text += "[%s unconfirmed]"%(format_satoshis(u, True, self.num_zeros).strip())
+                if x:
+                    text += "[%s unmatured]"%(format_satoshis(x, True, self.num_zeros).strip())
         else:
             self.status_image.set_from_stock(Gtk.STOCK_NO, Gtk.IconSize.MENU)
             self.network_button.set_tooltip_text("Not connected.")
             text = "Not connected"
 
-        self.status_bar.pop(self.context_id) 
+        self.status_bar.pop(self.context_id)
         self.status_bar.push(self.context_id, text)
 
         if self.wallet.up_to_date and self.wallet_updated:
@@ -1160,33 +1152,26 @@ class ElectrumWindow:
             if self.wallet.is_change(address): Type = "C"
             if address in self.wallet.imported_keys.keys():
                 Type = "I"
-            c, u = self.wallet.get_addr_balance(address)
-            if address in self.wallet.frozen_addresses: Type = Type + "F"
+            c, u, x = self.wallet.get_addr_balance(address)
+            if self.wallet.is_frozen(address): Type = Type + "F"
             label = self.wallet.labels.get(address)
             h = self.wallet.history.get(address,[])
             n = len(h)
             tx = "0" if n==0 else "%d"%n
-            self.recv_list.append((address, label, tx, format_satoshis(c,False,self.num_zeros), Type ))
+            self.recv_list.append((address, label, tx, format_satoshis(c+u+x, False, self.num_zeros), Type ))
 
     def update_sending_tab(self):
-        # detect addresses that are not mine in history, add them here...
         self.addressbook_list.clear()
-        #for alias, v in self.wallet.aliases.items():
-        #    s, target = v
-        #    label = self.wallet.labels.get(alias)
-        #    self.addressbook_list.append((alias, label, '-'))
-            
-        for address in self.wallet.addressbook:
-            label = self.wallet.labels.get(address)
-            n = self.wallet.get_num_tx(address)
-            self.addressbook_list.append((address, label, "%d"%n))
+        for k, v in self.contacts.items():
+            t, v = v
+            self.addressbook_list.append((k, v, t))
 
     def update_history_tab(self):
         cursor = self.history_treeview.get_cursor()[0]
         self.history_list.clear()
 
-        for item in self.wallet.get_tx_history():
-            tx_hash, conf, is_mine, value, fee, balance, timestamp = item
+        for item in self.wallet.get_history():
+            tx_hash, conf, value, timestamp, balance = item
             if conf > 0:
                 try:
                     time_str = datetime.datetime.fromtimestamp( timestamp).isoformat(' ')[:-3]
@@ -1214,8 +1199,9 @@ class ElectrumWindow:
         import datetime
         if not tx_hash: return ''
         tx = self.wallet.transactions.get(tx_hash)
-        is_relevant, is_mine, v, fee = self.wallet.get_tx_value(tx)
-        conf, timestamp = self.wallet.verifier.get_confirmations(tx_hash)
+        tx.deserialize()
+        is_relevant, is_mine, v, fee = self.wallet.get_wallet_delta(tx)
+        conf, timestamp = self.wallet.get_confirmations(tx_hash)
 
         if timestamp:
             time_str = datetime.datetime.fromtimestamp(timestamp).isoformat(' ')[:-3]
@@ -1223,12 +1209,12 @@ class ElectrumWindow:
             time_str = 'pending'
 
         inputs = map(lambda x: x.get('address'), tx.inputs)
-        outputs = map(lambda x: x.get('address'), tx.d['outputs'])
+        outputs = map(lambda x: x[0], tx.get_outputs())
         tx_details = "Transaction Details" +"\n\n" \
             + "Transaction ID:\n" + tx_hash + "\n\n" \
             + "Status: %d confirmations\n"%conf
         if is_mine:
-            if fee: 
+            if fee:
                 tx_details += "Amount sent: %s\n"% format_satoshis(v-fee, False) \
                               + "Transaction fee: %s\n"% format_satoshis(fee, False)
             else:
@@ -1247,8 +1233,8 @@ class ElectrumWindow:
 
     def newaddress_dialog(self, w):
 
-        title = "New Contact" 
-        dialog = Gtk.Dialog(title, parent=self.window, 
+        title = "New Contact"
+        dialog = Gtk.Dialog(title, parent=self.window,
                             flags=Gtk.DialogFlags.MODAL,
                             buttons= ("cancel", 0, "ok",1)  )
         dialog.show()
@@ -1274,7 +1260,7 @@ class ElectrumWindow:
         address.pack_start(address_entry, True, True, 0)
         address.show()
         dialog.vbox.pack_start(address, False, True, 5)
-        
+
         result = dialog.run()
         address = address_entry.get_text()
         label = label_entry.get_text()
@@ -1282,30 +1268,30 @@ class ElectrumWindow:
 
         if result == 1:
             if is_valid(address):
-                self.wallet.add_contact(address,label)
+                self.contacts[label] = address
                 self.update_sending_tab()
             else:
                 errorDialog = Gtk.MessageDialog(
                     parent=self.window,
-                    flags=Gtk.DialogFlags.MODAL, 
-                    buttons= Gtk.ButtonsType.CLOSE, 
+                    flags=Gtk.DialogFlags.MODAL,
+                    buttons= Gtk.ButtonsType.CLOSE,
                     message_format = "Invalid address")
                 errorDialog.show()
                 errorDialog.run()
                 errorDialog.destroy()
 
-    
+
 
 class ElectrumGui():
 
-    def __init__(self, config, network):
+    def __init__(self, config, network, plugins):
         self.network = network
         self.config = config
 
 
-    def main(self, url=None):
+    def main(self):
 
-        storage = WalletStorage(self.config)
+        storage = WalletStorage(self.config.get_wallet_path())
         if not storage.file_exists:
             action = self.restore_or_create()
             if not action:
@@ -1316,22 +1302,26 @@ class ElectrumGui():
                 wallet.gap_limit = gap
                 wallet.storage.put('gap_limit', gap, True)
 
-
             if action == 'create':
-                wallet.init_seed(None)
-                show_seed_dialog(wallet, None, None)
+                seed = wallet.make_seed()
+                show_seed_dialog(seed, None)
                 r = change_password_dialog(False, None)
                 password = r[2] if r else None
-                wallet.save_seed(password)
+                wallet.add_seed(seed, password)
+                wallet.create_master_keys(password)
+                wallet.create_main_account(password)
                 wallet.synchronize()  # generate first addresses offline
 
             elif action == 'restore':
                 seed = self.seed_dialog()
-                wallet.init_seed(seed)
+                if not seed:
+                    exit()
                 r = change_password_dialog(False, None)
                 password = r[2] if r else None
-                wallet.save_seed(password)
-                
+                wallet.add_seed(seed, password)
+                wallet.create_master_keys(password)
+                wallet.create_main_account(password)
+
             else:
                 exit()
         else:
@@ -1344,7 +1334,7 @@ class ElectrumGui():
             self.restore_wallet(wallet)
 
         w = ElectrumWindow(self.wallet, self.config, self.network)
-        if url: w.set_url(url)
+        #if url: w.set_url(url)
         Gtk.main()
 
     def restore_or_create(self):
@@ -1361,14 +1351,14 @@ class ElectrumGui():
 
         dialog = Gtk.MessageDialog(
             parent = None,
-            flags = Gtk.DialogFlags.MODAL, 
-            buttons = Gtk.ButtonsType.CANCEL, 
+            flags = Gtk.DialogFlags.MODAL,
+            buttons = Gtk.ButtonsType.CANCEL,
             message_format = "Please wait..."  )
         dialog.show()
 
         def recover_thread( wallet, dialog ):
-            wallet.restore(lambda x:x)
-            GObject.idle_add( dialog.destroy )
+            wallet.wait_until_synchronized()
+            GObject.idle_add(dialog.destroy)
 
         thread.start_new_thread( recover_thread, ( wallet, dialog ) )
         r = dialog.run()
