@@ -20,10 +20,12 @@ from PyQt4.QtGui import *
 from PyQt4.QtCore import *
 from electrum.i18n import _
 from util import *
+import re
+import math
 
 
 
-def make_password_dialog(self, wallet, msg):
+def make_password_dialog(self, wallet, msg, new_pass=True):
 
     self.pw = QLineEdit()
     self.pw.setEchoMode(2)
@@ -31,7 +33,7 @@ def make_password_dialog(self, wallet, msg):
     self.new_pw.setEchoMode(2)
     self.conf_pw = QLineEdit()
     self.conf_pw.setEchoMode(2)
-    
+
     vbox = QVBoxLayout()
     label = QLabel(msg)
     label.setWordWrap(True)
@@ -42,7 +44,7 @@ def make_password_dialog(self, wallet, msg):
     grid.setColumnStretch(1,1)
 
     logo = QLabel()
-    lockfile = ":icons/lock.png" if wallet.use_encryption else ":icons/unlock.png"
+    lockfile = ":icons/lock.png" if wallet and wallet.use_encryption else ":icons/unlock.png"
     logo.setPixmap(QPixmap(lockfile).scaledToWidth(36))
     logo.setAlignment(Qt.AlignCenter)
 
@@ -54,33 +56,38 @@ def make_password_dialog(self, wallet, msg):
     grid.setSpacing(8)
     grid.setColumnMinimumWidth(0, 250)
     grid.setColumnStretch(1,1)
-    
-    if wallet.use_encryption:
+
+    if wallet and wallet.use_encryption:
         grid.addWidget(QLabel(_('Password')), 0, 0)
         grid.addWidget(self.pw, 0, 1)
-        
-    grid.addWidget(QLabel(_('New Password')), 1, 0)
+
+    grid.addWidget(QLabel(_('New Password') if new_pass else _('Password')), 1, 0)
     grid.addWidget(self.new_pw, 1, 1)
 
     grid.addWidget(QLabel(_('Confirm Password')), 2, 0)
     grid.addWidget(self.conf_pw, 2, 1)
     vbox.addLayout(grid)
 
+    #Password Strength Label
+    self.pw_strength = QLabel()
+    grid.addWidget(self.pw_strength, 3, 0, 1, 2)
+    self.new_pw.textChanged.connect(lambda: update_password_strength(self.pw_strength, self.new_pw.text()))
+
     vbox.addStretch(1)
-    vbox.addLayout(ok_cancel_buttons(self))
+    vbox.addLayout(Buttons(CancelButton(self), OkButton(self)))
     return vbox
 
 
 def run_password_dialog(self, wallet, parent):
-        
-    if not wallet.seed:
-        QMessageBox.information(parent, _('Error'), _('No seed'), _('OK'))
+
+    if wallet and wallet.is_watching_only():
+        QMessageBox.information(parent, _('Error'), _('This is a watching-only wallet'), _('OK'))
         return False, None, None
 
     if not self.exec_():
         return False, None, None
 
-    password = unicode(self.pw.text()) if wallet.use_encryption else None
+    password = unicode(self.pw.text()) if wallet and wallet.use_encryption else None
     new_password = unicode(self.new_pw.text())
     new_password2 = unicode(self.conf_pw.text())
 
@@ -94,6 +101,39 @@ def run_password_dialog(self, wallet, parent):
 
     return True, password, new_password
 
+def check_password_strength(password):
+
+    '''
+    Check the strength of the password entered by the user and return back the same
+    :param password: password entered by user in New Password
+    :return: password strength Weak or Medium or Strong
+    '''
+    password = unicode(password)
+    n = math.log(len(set(password)))
+    num = re.search("[0-9]", password) is not None and re.match("^[0-9]*$", password) is None
+    caps = password != password.upper() and password != password.lower()
+    extra = re.match("^[a-zA-Z0-9]*$", password) is None
+    score = len(password)*( n + caps + num + extra)/20
+    password_strength = {0:"Weak",1:"Medium",2:"Strong",3:"Very Strong"}
+    return password_strength[min(3, int(score))]
+
+
+def update_password_strength(pw_strength_label,password):
+
+    '''
+    call the function check_password_strength and update the label pw_strength interactively as the user is typing the password
+    :param pw_strength_label: the label pw_strength
+    :param password: password entered in New Password text box
+    :return: None
+    '''
+    if password:
+        colors = {"Weak":"Red","Medium":"Blue","Strong":"Green", "Very Strong":"Green"}
+        strength = check_password_strength(password)
+        label = _("Password Strength")+ ": "+"<font color=" + colors[strength] + ">" + strength + "</font>"
+    else:
+        label = ""
+    pw_strength_label.setText(label)
+
 
 
 class PasswordDialog(QDialog):
@@ -103,6 +143,7 @@ class PasswordDialog(QDialog):
         self.setModal(1)
         self.wallet = wallet
         self.parent = parent
+        self.setWindowTitle(_("Set Password"))
         msg = (_('Your wallet is encrypted. Use this dialog to change your password.') + ' '\
                +_('To disable wallet encryption, enter an empty new password.')) \
                if wallet.use_encryption else _('Your wallet keys are not encrypted')
@@ -115,14 +156,16 @@ class PasswordDialog(QDialog):
             return
 
         try:
-            self.wallet.get_seed(password)
-        except Exception:
-            QMessageBox.warning(self.parent, _('Error'), _('Incorrect Password'), _('OK'))
+            self.wallet.check_password(password)
+        except BaseException as e:
+            QMessageBox.warning(self.parent, _('Error'), str(e), _('OK'))
             return False, None, None
 
         try:
             self.wallet.update_password(password, new_password)
         except:
+            import traceback, sys
+            traceback.print_exc(file=sys.stdout)
             QMessageBox.warning(self.parent, _('Error'), _('Failed to update password'), _('OK'))
             return
 
@@ -130,7 +173,3 @@ class PasswordDialog(QDialog):
             QMessageBox.information(self.parent, _('Success'), _('Password was updated successfully'), _('OK'))
         else:
             QMessageBox.information(self.parent, _('Success'), _('This wallet is not encrypted'), _('OK'))
-
-
-
-
